@@ -11,6 +11,8 @@ import sys
 import typing
 from pathlib import Path
 from sysconfig import get_paths, get_python_version, get_scheme_names
+from urllib.parse import urlparse
+from urllib.request import url2pathname
 
 import pipenv
 from pipenv.patched.pip._internal.commands.install import InstallCommand
@@ -36,7 +38,7 @@ except ImportError:
 
 if typing.TYPE_CHECKING:
     from types import ModuleType
-    from typing import ContextManager, Dict, Generator, List, Optional, Set, Union
+    from typing import ContextManager, Generator
 
     from pipenv.project import Project, TPipfile, TSource
     from pipenv.vendor import tomlkit
@@ -47,13 +49,13 @@ BASE_WORKING_SET = pkg_resources.WorkingSet(sys.path)
 class Environment:
     def __init__(
         self,
-        prefix: Optional[str] = None,
-        python: Optional[str] = None,
+        prefix: str | None = None,
+        python: str | None = None,
         is_venv: bool = False,
         base_working_set: pkg_resources.WorkingSet = None,
-        pipfile: Optional[Union[tomlkit.toml_document.TOMLDocument, TPipfile]] = None,
-        sources: Optional[List[TSource]] = None,
-        project: Optional[Project] = None,
+        pipfile: tomlkit.toml_document.TOMLDocument | TPipfile | None = None,
+        sources: list[TSource] | None = None,
+        project: Project | None = None,
     ):
         super().__init__()
         self._modules = {"pkg_resources": pkg_resources, "pipenv": pipenv}
@@ -99,7 +101,7 @@ class Environment:
     @classmethod
     def resolve_dist(
         cls, dist: pkg_resources.Distribution, working_set: pkg_resources.WorkingSet
-    ) -> Set[pkg_resources.Distribution]:
+    ) -> set[pkg_resources.Distribution]:
         """Given a local distribution and a working set, returns all dependencies from the set.
 
         :param dist: A single distribution to find the dependencies of
@@ -144,12 +146,12 @@ class Environment:
             py_version = sysconfig.get_python_version()
             return py_version
 
-    def find_libdir(self) -> Optional[Path]:
+    def find_libdir(self) -> Path | None:
         libdir = self.prefix / "lib"
         return next(iter(list(libdir.iterdir())), None)
 
     @property
-    def python_info(self) -> Dict[str, str]:
+    def python_info(self) -> dict[str, str]:
         include_dir = self.prefix / "include"
         if not os.path.exists(include_dir):
             include_dirs = self.get_include_path()
@@ -188,7 +190,7 @@ class Environment:
             return "posix_prefix"
 
     @cached_property
-    def base_paths(self) -> Dict[str, str]:
+    def base_paths(self) -> dict[str, str]:
         """
         Returns the context appropriate paths for the environment.
 
@@ -299,7 +301,7 @@ class Environment:
         return py
 
     @cached_property
-    def sys_path(self) -> List[str]:
+    def sys_path(self) -> list[str]:
         """
         The system path inside the environment
 
@@ -370,7 +372,7 @@ class Environment:
         py_command = py_command % lines_as_str
         return py_command
 
-    def get_paths(self) -> Optional[Dict[str, str]]:
+    def get_paths(self) -> dict[str, str] | None:
         """
         Get the paths for the environment by running a subcommand
 
@@ -402,7 +404,7 @@ class Environment:
             click.secho(f"Output: {c.stdout}", fg="yellow")
         return None
 
-    def get_lib_paths(self) -> Dict[str, str]:
+    def get_lib_paths(self) -> dict[str, str]:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -453,7 +455,7 @@ class Environment:
                 return paths
         return {}
 
-    def get_include_path(self) -> Optional[Dict[str, str]]:
+    def get_include_path(self) -> dict[str, str] | None:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -488,7 +490,7 @@ class Environment:
         return sys_prefix
 
     @cached_property
-    def paths(self) -> Dict[str, str]:
+    def paths(self) -> dict[str, str]:
         paths = {}
         with temp_environ(), temp_path():
             os.environ["PYTHONIOENCODING"] = "utf-8"
@@ -540,12 +542,7 @@ class Environment:
         :rtype: iterator
         """
 
-        pip_target_dir = os.environ.get("PIP_TARGET")
-        libdirs = (
-            [pip_target_dir]
-            if pip_target_dir
-            else self.base_paths["libdirs"].split(os.pathsep)
-        )
+        libdirs = self.base_paths["libdirs"].split(os.pathsep)
         dists = (pkg_resources.find_distributions(libdir) for libdir in libdirs)
         yield from itertools.chain.from_iterable(dists)
 
@@ -586,7 +583,7 @@ class Environment:
         location = _normalized(make_posix(location))
         return any(location.startswith(prefix) for prefix in prefixes)
 
-    def get_installed_packages(self) -> List[pkg_resources.Distribution]:
+    def get_installed_packages(self) -> list[pkg_resources.Distribution]:
         """Returns all of the installed packages in a given environment"""
         workingset = self.get_working_set()
         packages = [
@@ -650,7 +647,7 @@ class Environment:
 
     def get_outdated_packages(
         self, pre: bool = False
-    ) -> List[pkg_resources.Distribution]:
+    ) -> list[pkg_resources.Distribution]:
         return [
             pkg
             for pkg in self.get_package_info(pre=pre)
@@ -775,7 +772,12 @@ class Environment:
         if match is not None:
             if req.editable and req.line_instance.is_local and self.find_egg(match):
                 requested_path = req.line_instance.path
-                return requested_path and os.path.samefile(requested_path, match.location)
+                if os.path.exists(requested_path):
+                    local_path = requested_path
+                else:
+                    parsed_url = urlparse(requested_path)
+                    local_path = url2pathname(parsed_url.path)
+                return requested_path and os.path.samefile(local_path, match.location)
             elif match.has_metadata("direct_url.json"):
                 direct_url_metadata = json.loads(match.get_metadata("direct_url.json"))
                 commit_id = direct_url_metadata.get("vcs_info", {}).get("commit_id", "")
