@@ -8,6 +8,15 @@ os.environ["PIP_PYTHON_PATH"] = str(sys.executable)
 
 
 def _ensure_modules():
+    spec = importlib.util.spec_from_file_location(
+        "typing_extensions",
+        location=os.path.join(
+            os.path.dirname(__file__), "patched", "pip", "_vendor", "typing_extensions.py"
+        ),
+    )
+    typing_extensions = importlib.util.module_from_spec(spec)
+    sys.modules["typing_extensions"] = typing_extensions
+    spec.loader.exec_module(typing_extensions)
     spec = importlib.util.spec_from_file_location(
         "pipenv", location=os.path.join(os.path.dirname(__file__), "__init__.py")
     )
@@ -131,11 +140,9 @@ class Entry:
         self._parents_in_pipfile = []
 
     @staticmethod
-    def make_requirement(name=None, entry=None, from_ireq=False):
+    def make_requirement(name=None, entry=None):
         from pipenv.vendor.requirementslib.models.requirements import Requirement
 
-        if from_ireq:
-            return Requirement.from_ireq(entry)
         return Requirement.from_pipfile(name, entry)
 
     @classmethod
@@ -148,7 +155,7 @@ class Entry:
 
     @classmethod
     def parse_pyparsing_exprs(cls, expr_iterable):
-        from pipenv.vendor.pyparsing import Literal, MatchFirst
+        from pipenv.patched.pip._vendor.pyparsing import Literal, MatchFirst
 
         keys = []
         expr_list = []
@@ -195,7 +202,7 @@ class Entry:
         if not markers:
             marker_str = self.marker_to_str(markers)
             if marker_str:
-                self._entry = self.entry.merge_markers(marker_str)
+                self.entry.merge_markers(marker_str)
                 self._markers = self.marker_to_str(self._entry.markers)
                 entry_dict = self.entry_dict.copy()
                 entry_dict["markers"] = self.marker_to_str(self._entry.markers)
@@ -229,12 +236,8 @@ class Entry:
             return marker_str
         return None
 
-    def get_cleaned_dict(self, keep_outdated=False):
-        if keep_outdated and self.is_updated:
-            self.validate_constraints()
-            self.ensure_least_updates_possible()
-        elif not keep_outdated:
-            self.validate_constraints()
+    def get_cleaned_dict(self):
+        self.validate_constraints()
         if self.entry.extras != self.lockfile_entry.extras:
             entry_extras = list(self.entry.extras)
             if self.lockfile_entry.extras:
@@ -441,58 +444,6 @@ class Entry:
                 parents.extend(parent.flattened_parents)
         return parents
 
-    def ensure_least_updates_possible(self):
-        """
-        Mutate the current entry to ensure that we are making the smallest amount of
-        changes possible to the existing lockfile -- this will keep the old locked
-        versions of packages if they satisfy new constraints.
-
-        :return: None
-        """
-        constraints = self.get_constraints()
-        can_use_original = True
-        can_use_updated = True
-        satisfied_by_versions = set()
-        for constraint in constraints:
-            if not constraint.specifier.contains(self.original_version):
-                self.can_use_original = False
-            if not constraint.specifier.contains(self.updated_version):
-                self.can_use_updated = False
-            satisfied_by_value = getattr(constraint, "satisfied_by", None)
-            if satisfied_by_value:
-                satisfied_by = "{}".format(
-                    self.clean_specifier(str(satisfied_by_value.version))
-                )
-                satisfied_by_versions.add(satisfied_by)
-        if can_use_original:
-            self.entry_dict = self.lockfile_dict.copy()
-        elif can_use_updated:
-            if len(satisfied_by_versions) == 1:
-                self.entry_dict["version"] = next(
-                    iter(sat_by for sat_by in satisfied_by_versions if sat_by), None
-                )
-                hashes = None
-                if self.lockfile_entry.specifiers == satisfied_by:
-                    ireq = self.lockfile_entry.as_ireq()
-                    if (
-                        not self.lockfile_entry.hashes
-                        and self.resolver._should_include_hash(ireq)
-                    ):
-                        hashes = self.resolver.get_hash(ireq)
-                    else:
-                        hashes = self.lockfile_entry.hashes
-                else:
-                    if self.resolver._should_include_hash(constraint):
-                        hashes = self.resolver.get_hash(constraint)
-                if hashes:
-                    self.entry_dict["hashes"] = list(hashes)
-                    self._entry.hashes = frozenset(hashes)
-        else:
-            # check for any parents, since they depend on this and the current
-            # installed versions are not compatible with the new version, so
-            # we will need to update the top level dependency if possible
-            self.check_flattened_parents()
-
     def get_constraints(self):
         """
         Retrieve all of the relevant constraints, aggregated from the pipfile, resolver,
@@ -519,7 +470,7 @@ class Entry:
         :return: An **InstallRequirement** instance representing a version constraint
         """
         if self.is_in_pipfile:
-            return self.pipfile_entry.as_ireq()
+            return self.pipfile_entry.ireq
 
     def validate_constraints(self):
         """
@@ -626,73 +577,15 @@ def clean_results(results, resolver, project, category):
             reverse_deps=reverse_deps,
             category=category,
         )
-        entry_dict = translate_markers(entry.get_cleaned_dict(keep_outdated=False))
-        new_results.append(entry_dict)
-    return new_results
-
-
-def clean_outdated(results, resolver, project, category):
-    from pipenv.utils.dependencies import get_lockfile_section_using_pipfile_category
-
-    if not project.lockfile_exists:
-        return results
-    lockfile = project.lockfile_content
-    lockfile_section = get_lockfile_section_using_pipfile_category(category)
-    reverse_deps = project.environment.reverse_dependencies()
-    new_results = [r for r in results if r["name"] not in lockfile[lockfile_section]]
-    for result in results:
-        name = result.get("name")
-        entry_dict = result.copy()
-        entry = Entry(
-            name,
-            entry_dict,
-            project,
-            resolver,
-            reverse_deps=reverse_deps,
-            category=category,
-        )
-        # The old entry was editable but this one isnt; prefer the old one
-        # TODO: Should this be the case for all locking?
-        if entry.was_editable and not entry.is_editable:
-            continue
-        lockfile_entry = lockfile[lockfile_section].get(name, None)
-        if not lockfile_entry:
-            if name in lockfile[lockfile_section]:
-                lockfile_entry = lockfile[lockfile_section][name]
-        if lockfile_entry and not entry.is_updated:
-            old_markers = next(
-                iter(
-                    m
-                    for m in (
-                        entry.lockfile_entry.markers,
-                        lockfile_entry.get("markers", None),
-                    )
-                    if m is not None
-                ),
-                None,
-            )
-            new_markers = entry_dict.get("markers", None)
-            if old_markers:
-                old_markers = Entry.marker_to_str(old_markers)
-            if old_markers and not new_markers:
-                entry.markers = old_markers
-            elif new_markers and not old_markers:
-                del entry.entry_dict["markers"]
-                entry._entry.req.req.marker = None
-                entry._entry.markers = None
-            # if the entry has not changed versions since the previous lock,
-            # don't introduce new markers since that is more restrictive
-            # if entry.has_markers and not entry.had_markers and not entry.is_updated:
-            # do make sure we retain the original markers for entries that are not changed
-        entry_dict = entry.get_cleaned_dict(keep_outdated=True)
+        entry_dict = translate_markers(entry.get_cleaned_dict())
         new_results.append(entry_dict)
     return new_results
 
 
 def parse_packages(packages, pre, clear, system, requirements_dir=None):
     from pipenv.utils.indexes import parse_indexes
+    from pipenv.vendor.requirementslib.fileutils import cd, temp_path
     from pipenv.vendor.requirementslib.models.requirements import Requirement
-    from pipenv.vendor.vistir.contextmanagers import cd, temp_path
 
     parsed_packages = []
     for package in packages:
@@ -703,7 +596,7 @@ def parse_packages(packages, pre, clear, system, requirements_dir=None):
         if not req.name:
             with temp_path(), cd(req.req.setup_info.base_dir):
                 sys.path.insert(0, req.req.setup_info.base_dir)
-                req.req._setup_info.get_info()
+                req.req.setup_info.get_info()
                 req.update_name_from_path(req.req.setup_info.base_dir)
         try:
             name, entry = req.pipfile_entry
@@ -721,7 +614,7 @@ def parse_packages(packages, pre, clear, system, requirements_dir=None):
 
 
 def resolve_packages(
-    pre, clear, verbose, system, write, requirements_dir, packages, category
+    pre, clear, verbose, system, requirements_dir, packages, category, constraints=None
 ):
     from pipenv.utils.internet import create_mirror_source, replace_pypi_sources
     from pipenv.utils.resolver import resolve_deps
@@ -732,6 +625,9 @@ def resolve_packages(
         else None
     )
 
+    if constraints:
+        packages += constraints
+
     def resolve(
         packages, pre, project, sources, clear, system, category, requirements_dir=None
     ):
@@ -755,7 +651,6 @@ def resolve_packages(
         if pypi_mirror_source
         else project.pipfile_sources()
     )
-    keep_outdated = os.environ.get("PIPENV_KEEP_OUTDATED", False)
     results, resolver = resolve(
         packages,
         pre=pre,
@@ -766,51 +661,10 @@ def resolve_packages(
         system=system,
         requirements_dir=requirements_dir,
     )
-    if keep_outdated:
-        results = clean_outdated(results, resolver, project, category)
-    else:
-        results = clean_results(results, resolver, project, category)
-    if write:
-        with open(write, "w") as fh:
-            if not results:
-                json.dump([], fh)
-            else:
-                json.dump(results, fh)
-    else:
-        print("RESULTS:")
-        if results:
-            print(json.dumps(results))
-        else:
-            print(json.dumps([]))
-
-
-def _main(
-    pre,
-    clear,
-    verbose,
-    system,
-    write,
-    requirements_dir,
-    packages,
-    parse_only=False,
-    category=None,
-):
-    os.environ["PIPENV_REQUESTED_PYTHON_VERSION"] = ".".join(
-        [str(s) for s in sys.version_info[:3]]
-    )
-    os.environ["PIP_PYTHON_PATH"] = str(sys.executable)
-    if parse_only:
-        parse_packages(
-            packages,
-            pre=pre,
-            clear=clear,
-            system=system,
-            requirements_dir=requirements_dir,
-        )
-    else:
-        resolve_packages(
-            pre, clear, verbose, system, write, requirements_dir, packages, category
-        )
+    results = clean_results(results, resolver, project, category)
+    if results:
+        return results
+    return []
 
 
 def main(argv=None):
@@ -828,15 +682,13 @@ def main(argv=None):
     os.environ["PYTHONIOENCODING"] = "utf-8"
     os.environ["PYTHONUNBUFFERED"] = "1"
     parsed = handle_parsed_args(parsed)
-    _main(
+    resolve_packages(
         parsed.pre,
         parsed.clear,
         parsed.verbose,
         parsed.system,
-        parsed.write,
         parsed.requirements_dir,
         parsed.packages,
-        parse_only=parsed.parse_only,
         category=parsed.category,
     )
 
