@@ -9,19 +9,15 @@ from pipenv import environments, exceptions
 from pipenv.patched.pip._internal.exceptions import PipError
 from pipenv.patched.pip._vendor import rich
 from pipenv.routines.lock import do_lock
-from pipenv.utils.dependencies import convert_deps_to_pip, is_star
 from pipenv.utils.indexes import get_source_list
-from pipenv.utils.internet import download_file, get_host_and_port, is_valid_url
+from pipenv.utils.internet import download_file, is_valid_url
 from pipenv.utils.pip import (
-    format_pip_error,
-    format_pip_output,
     get_trusted_hosts,
-    pip_install,
     pip_install_deps,
 )
 from pipenv.utils.pipfile import ensure_pipfile
 from pipenv.utils.project import ensure_project
-from pipenv.utils.requirements import import_requirements
+from pipenv.utils.requirements import add_index_to_pipfile, import_requirements
 from pipenv.utils.virtualenv import cleanup_virtualenv, do_create_virtualenv
 from pipenv.vendor import click
 from pipenv.vendor.requirementslib import fileutils
@@ -46,8 +42,6 @@ def do_install(
     requirementstxt=False,
     pre=False,
     deploy=False,
-    keep_outdated=False,
-    selective_upgrade=False,
     site_packages=None,
     extra_pip_args=None,
     categories=None,
@@ -56,8 +50,6 @@ def do_install(
         suffix="-requirements", prefix="pipenv-"
     )
     warnings.filterwarnings("default", category=ResourceWarning)
-    if selective_upgrade:
-        keep_outdated = True
     packages = packages if packages else []
     editable_packages = editable_packages if editable_packages else []
     package_args = [p for p in packages if p] + [p for p in editable_packages if p]
@@ -85,8 +77,6 @@ def do_install(
     # Load the --pre settings from the Pipfile.
     if not pre:
         pre = project.settings.get("allow_prereleases")
-    if not keep_outdated:
-        keep_outdated = project.settings.get("keep_outdated")
     remote = requirementstxt and is_valid_url(requirementstxt)
     if "default" in categories:
         raise exceptions.PipenvUsageError(
@@ -169,29 +159,12 @@ def do_install(
 
     # Allow more than one package to be provided.
     package_args = [p for p in packages] + [f"-e {pkg}" for pkg in editable_packages]
-    # Support for --selective-upgrade.
-    # We should do this part first to make sure that we actually do selectively upgrade
-    # the items specified
-    if selective_upgrade:
-        from pipenv.vendor.requirementslib.models.requirements import Requirement
-
-        for i, package in enumerate(package_args[:]):
-            section = project.packages if not dev else project.dev_packages
-            package = Requirement.from_line(package)
-            package__name, package__val = package.pipfile_entry
-            try:
-                if not is_star(section[package__name]) and is_star(package__val):
-                    # Support for VCS dependencies.
-                    package_args[i] = convert_deps_to_pip(
-                        {package__name: section[package__name]}, project=project
-                    )[0]
-            except KeyError:
-                pass
     # Install all dependencies, if none was provided.
     # This basically ensures that we have a pipfile and lockfile, then it locks and
     # installs from the lockfile
+    new_packages = []
     if not packages and not editable_packages:
-        # Update project settings with pre preference.
+        # Update project settings with prerelease preference.
         if pre:
             project.update_settings({"allow_prereleases": pre})
         do_init(
@@ -205,7 +178,6 @@ def do_install(
             pre=pre,
             requirements_dir=requirements_directory,
             pypi_mirror=pypi_mirror,
-            keep_outdated=keep_outdated,
             extra_pip_args=extra_pip_args,
             categories=categories,
         )
@@ -222,7 +194,6 @@ def do_install(
                 dev=dev,
                 system=system,
                 allow_global=system,
-                keep_outdated=keep_outdated,
                 requirements_dir=requirements_directory,
                 deploy=deploy,
                 pypi_mirror=pypi_mirror,
@@ -256,60 +227,7 @@ def do_install(
                         )
                     )
                     sys.exit(1)
-                st.console.print("Installing...")
-                try:
-                    st.update(f"Installing {pkg_requirement.name}...")
-                    if project.s.is_verbose():
-                        st.console.print(
-                            f"Installing package: {pkg_requirement.as_line(include_hashes=False)}"
-                        )
-                    c = pip_install(
-                        project,
-                        pkg_requirement,
-                        ignore_hashes=True,
-                        allow_global=system,
-                        selective_upgrade=selective_upgrade,
-                        no_deps=False,
-                        pre=pre,
-                        dev=dev,
-                        requirements_dir=requirements_directory,
-                        index=index_url,
-                        pypi_mirror=pypi_mirror,
-                        use_constraint=True,
-                        extra_pip_args=extra_pip_args,
-                    )
-                    if c.returncode:
-                        err.print(
-                            "{} An error occurred while installing {}!".format(
-                                click.style("Error: ", fg="red", bold=True),
-                                click.style(pkg_line, fg="green"),
-                            ),
-                        )
-                        err.print(f"Error text: {c.stdout}")
-                        err.print(click.style(format_pip_error(c.stderr), fg="cyan"))
-                        if project.s.is_verbose():
-                            err.print(click.style(format_pip_output(c.stdout), fg="cyan"))
-                        if "setup.py egg_info" in c.stderr:
-                            err.print(
-                                "This is likely caused by a bug in {}. "
-                                "Report this to its maintainers.".format(
-                                    click.style(pkg_requirement.name, fg="green")
-                                )
-                            )
-                        err.print(
-                            environments.PIPENV_SPINNER_FAIL_TEXT.format(
-                                "Installation Failed"
-                            )
-                        )
-                        sys.exit(1)
-                except (ValueError, RuntimeError) as e:
-                    err.print("{}: {}".format(click.style("WARNING", fg="red"), e))
-                    err.print(
-                        environments.PIPENV_SPINNER_FAIL_TEXT.format(
-                            "Installation Failed",
-                        )
-                    )
-                    sys.exit(1)
+                st.update(f"Installing {pkg_requirement.name}...")
                 # Warn if --editable wasn't passed.
                 if (
                     pkg_requirement.is_vcs
@@ -338,29 +256,20 @@ def do_install(
                 )
                 # Add the package to the Pipfile.
                 if index_url:
-                    trusted_hosts = get_trusted_hosts()
-                    host_and_port = get_host_and_port(index_url)
-                    require_valid_https = not any(
-                        (
-                            v in trusted_hosts
-                            for v in (
-                                host_and_port,
-                                host_and_port.partition(":")[
-                                    0
-                                ],  # also check if hostname without port is in trusted_hosts
-                            )
-                        )
-                    )
-                    index_name = project.add_index_to_pipfile(
-                        index_url, verify_ssl=require_valid_https
-                    )
+                    index_name = add_index_to_pipfile(project, index_url)
                     pkg_requirement.index = index_name
                 try:
                     if categories:
                         for category in categories:
-                            project.add_package_to_pipfile(pkg_requirement, dev, category)
+                            added, cat = project.add_package_to_pipfile(
+                                pkg_requirement, dev, category
+                            )
+                            if added:
+                                new_packages.append((pkg_requirement.name, cat))
                     else:
-                        project.add_package_to_pipfile(pkg_requirement, dev)
+                        added, cat = project.add_package_to_pipfile(pkg_requirement, dev)
+                        if added:
+                            new_packages.append((pkg_requirement.name, cat))
                 except ValueError:
                     import traceback
 
@@ -375,26 +284,31 @@ def do_install(
                             "Failed adding package to Pipfile"
                         )
                     )
-                # ok has a nice v in front, should do something similir with rich
+                # ok has a nice v in front, should do something similar with rich
                 st.console.print(
                     environments.PIPENV_SPINNER_OK_TEXT.format("Installation Succeeded")
                 )
             # Update project settings with pre-release preference.
             if pre:
                 project.update_settings({"allow_prereleases": pre})
-        do_init(
-            project,
-            dev=dev,
-            system=system,
-            allow_global=system,
-            keep_outdated=keep_outdated,
-            requirements_dir=requirements_directory,
-            deploy=deploy,
-            pypi_mirror=pypi_mirror,
-            skip_lock=skip_lock,
-            extra_pip_args=extra_pip_args,
-            categories=categories,
-        )
+        try:
+            do_init(
+                project,
+                dev=dev,
+                system=system,
+                allow_global=system,
+                requirements_dir=requirements_directory,
+                deploy=deploy,
+                pypi_mirror=pypi_mirror,
+                skip_lock=skip_lock,
+                extra_pip_args=extra_pip_args,
+                categories=categories,
+            )
+        except RuntimeError:
+            # If we fail to install, remove the package from the Pipfile.
+            for pkg_name, category in new_packages:
+                project.remove_package_from_pipfile(pkg_name, category)
+            sys.exit(1)
     sys.exit(0)
 
 
@@ -403,7 +317,6 @@ def do_sync(
     dev=False,
     python=None,
     bare=False,
-    dont_upgrade=False,
     user=False,
     clear=False,
     unused=False,
@@ -412,6 +325,7 @@ def do_sync(
     deploy=False,
     extra_pip_args=None,
     categories=None,
+    site_packages=False,
 ):
     # The lock file needs to exist because sync won't write to it.
     if not project.lockfile_exists:
@@ -426,6 +340,7 @@ def do_sync(
         deploy=deploy,
         pypi_mirror=pypi_mirror,
         clear=clear,
+        site_packages=site_packages,
     )
 
     # Install everything.
@@ -585,7 +500,7 @@ def batch_install_iteration(
         if dep.req.req:
             dep.req.req = strip_extras_markers_from_requirement(dep.req.req)
         if dep.markers:
-            dep.markers = str(strip_extras_markers_from_requirement(dep.get_markers()))
+            dep.markers = str(strip_extras_markers_from_requirement(dep.get_markers))
         # Install the module.
         if dep.is_file_or_url and (
             dep.is_direct_url
@@ -744,8 +659,6 @@ def _cleanup_procs(project, procs, failed_deps_queue, retry=True):
                     raise exceptions.InstallError(deps, extra=err_lines)
                 else:
                     # Alert the user.
-                    if dep:
-                        dep.use_pep517 = False
                     click.echo(
                         "{} {}! Will try again.".format(
                             click.style("An error occurred while installing", fg="red"),
@@ -767,7 +680,6 @@ def do_init(
     system=False,
     deploy=False,
     pre=False,
-    keep_outdated=False,
     requirements_dir=None,
     pypi_mirror=None,
     extra_pip_args=None,
@@ -833,7 +745,6 @@ def do_init(
                     project,
                     system=system,
                     pre=pre,
-                    keep_outdated=keep_outdated,
                     write=True,
                     pypi_mirror=pypi_mirror,
                     categories=categories,
@@ -859,7 +770,6 @@ def do_init(
                 project,
                 system=system,
                 pre=pre,
-                keep_outdated=keep_outdated,
                 write=True,
                 pypi_mirror=pypi_mirror,
                 categories=categories,
