@@ -1,14 +1,11 @@
 import contextlib
 import hashlib
-import json
 import os
 import subprocess
 import sys
-import tempfile
 import warnings
 from functools import lru_cache
 from html.parser import HTMLParser
-from pathlib import Path
 from typing import Dict, List, Optional, Set, Tuple, Union
 from urllib import parse
 
@@ -30,10 +27,10 @@ from pipenv.patched.pip._internal.utils.hashes import FAVORITE_HASH
 from pipenv.patched.pip._internal.utils.temp_dir import global_tempdir_manager
 from pipenv.patched.pip._vendor import pkg_resources, rich
 from pipenv.project import Project
+from pipenv.resolver import resolve_packages
 from pipenv.vendor import click
-from pipenv.vendor.requirementslib import Requirement
 from pipenv.vendor.requirementslib.fileutils import create_tracked_tempdir, open_file
-from pipenv.vendor.requirementslib.models.requirements import Line
+from pipenv.vendor.requirementslib.models.requirements import Line, Requirement
 from pipenv.vendor.requirementslib.models.utils import DIRECT_URL_RE
 
 try:
@@ -58,7 +55,7 @@ from .dependencies import (
 from .indexes import parse_indexes, prepare_pip_source_args
 from .internet import _get_requests_session, is_pypi_url
 from .locking import format_requirement_for_lockfile, prepare_lockfile
-from .shell import make_posix, subprocess_run, temp_environ
+from .shell import subprocess_run, temp_environ
 
 console = rich.console.Console()
 err = rich.console.Console(stderr=True)
@@ -122,7 +119,7 @@ class HashCacheMixin:
         with open_file(link.url, self.session) as fp:
             for chunk in iter(lambda: fp.read(8096), b""):
                 h.update(chunk)
-        return ":".join([h.name, h.hexdigest()])
+        return f"{h.name}:{h.hexdigest()}"
 
 
 class PackageIndexHTMLParser(HTMLParser):
@@ -283,13 +280,13 @@ class Resolver:
             project = Project()
         index, extra_index, trust_host, remainder = parse_indexes(line)
         line = " ".join(remainder)
-        req: Requirement = None
         try:
             req = Requirement.from_line(line)
         except ValueError:
             direct_url = DIRECT_URL_RE.match(line)
             if direct_url:
-                line = "{}#egg={}".format(line, direct_url.groupdict()["name"])
+                name = direct_url.groupdict()["name"]
+                line = f"{name}@ {line}"
                 try:
                     req = Requirement.from_line(line)
                 except ValueError:
@@ -347,8 +344,10 @@ class Resolver:
                 name = req.normalized_name
             except TypeError:
                 raise RequirementError(req=req)
-            setup_info = req.req.setup_info
-            setup_info.get_info()
+            if parsed_line.setup_info:
+                setup_info = parsed_line.setup_info
+            else:
+                setup_info = req.req.parse_setup_info()
             locked_deps[pep423_name(name)] = entry
             requirements = []
             # Allow users to toggle resolution off for non-editable VCS packages
@@ -362,7 +361,16 @@ class Resolver:
                     and is_installable_dir(parsed_line.path)
                 )
             ):
+                setup_info.run_pyproject()
+                setup_info.run_setup()
                 requirements = [v for v in getattr(setup_info, "requires", {}).values()]
+                if req.extras:
+                    for extra in req.extras:
+                        requirements.extend(
+                            v
+                            for v in getattr(setup_info, "extras", {}).get(extra, [])
+                            if v not in requirements
+                        )
             for r in requirements:
                 if getattr(r, "url", None) and not getattr(r, "editable", False):
                     if r is not None:
@@ -417,8 +425,8 @@ class Resolver:
                     )
                     hashes = resolver.collect_hashes(ireq) if resolver else []
                     new_req = Requirement.from_ireq(ireq)
-                    new_req = new_req.add_hashes(hashes)
-                    new_req = new_req.merge_markers(req.markers)
+                    new_req.add_hashes(hashes)
+                    new_req.merge_markers(req.markers)
                     name, entry = new_req.pipfile_entry
                     locked_deps[pep423_name(name)] = translate_markers(entry)
                     click.echo(
@@ -666,7 +674,9 @@ class Resolver:
     def get_resolver(self, clear=False):
         from pipenv.patched.pip._internal.utils.temp_dir import TempDirectory
 
-        with global_tempdir_manager(), get_build_tracker() as build_tracker, TempDirectory() as directory:
+        with global_tempdir_manager(), get_build_tracker() as build_tracker, TempDirectory(
+            globally_managed=True
+        ) as directory:
             pip_options = self.pip_options
             finder = self.finder
             wheel_cache = WheelCache(pip_options.cache_dir)
@@ -876,7 +886,7 @@ class Resolver:
         ref = None
         if req.is_vcs:
             ref = req.commit_hash
-        ireq = req.as_ireq()
+        ireq = req.ireq
         entry = value.copy()
         entry["name"] = req.name
         if entry.get("editable", False) and entry.get("version"):
@@ -890,8 +900,6 @@ class Resolver:
         return req.name, entry
 
     def clean_results(self):
-        from pipenv.vendor.requirementslib.models.requirements import Requirement
-
         reqs = [(Requirement.from_ireq(ireq), ireq) for ireq in self.resolved_tree]
         results = {}
         for req, ireq in reqs:
@@ -900,7 +908,7 @@ class Resolver:
             elif req.normalized_name in self.skipped.keys():
                 continue
             collected_hashes = self.hashes.get(ireq, set())
-            req = req.add_hashes(collected_hashes)
+            req.add_hashes(collected_hashes)
             if collected_hashes:
                 collected_hashes = sorted(collected_hashes)
             name, entry = format_requirement_for_lockfile(
@@ -944,11 +952,8 @@ def actually_resolve_deps(
     clear,
     pre,
     category,
-    req_dir=None,
+    req_dir,
 ):
-    if not req_dir:
-        req_dir = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
-
     with warnings.catch_warnings(record=True) as warning_list:
         resolver = Resolver.create(
             deps,
@@ -1015,7 +1020,6 @@ def venv_resolve_deps(
     pypi_mirror=None,
     pipfile=None,
     lockfile=None,
-    keep_outdated=False,
 ):
     """
     Resolve dependencies for a pipenv project, acts as a portal to the target environment.
@@ -1036,13 +1040,10 @@ def venv_resolve_deps(
     :param pipfile: A Pipfile section to operate on, defaults to None
     :type pipfile: Optional[Dict[str, Union[str, Dict[str, bool, List[str]]]]]
     :param Dict[str, Any] lockfile: A project lockfile to mutate, defaults to None
-    :param bool keep_outdated: Whether to retain outdated dependencies and resolve with them in mind, defaults to False
     :raises RuntimeError: Raised on resolution failure
     :return: The lock data
     :rtype: dict
     """
-    from pipenv import resolver
-
     lockfile_section = get_lockfile_section_using_pipfile_category(category)
 
     if not deps:
@@ -1055,77 +1056,48 @@ def venv_resolve_deps(
     if not pipfile:
         pipfile = getattr(project, category, {})
     if lockfile is None:
-        lockfile = project._lockfile(categories=[category])
+        lockfile = project.lockfile(categories=[category])
     req_dir = create_tracked_tempdir(prefix="pipenv", suffix="requirements")
-    cmd = [
-        which("python", allow_global=allow_global),
-        Path(resolver.__file__.rstrip("co")).as_posix(),
-    ]
-    if pre:
-        cmd.append("--pre")
-    if clear:
-        cmd.append("--clear")
-    if allow_global:
-        cmd.append("--system")
-    if category:
-        cmd.append("--category")
-        cmd.append(category)
-    target_file = tempfile.NamedTemporaryFile(
-        prefix="resolver", suffix=".json", delete=False
-    )
-    target_file.close()
-    cmd.extend(["--write", make_posix(target_file.name)])
+    results = []
     with temp_environ():
         os.environ.update({k: str(val) for k, val in os.environ.items()})
         if pypi_mirror:
             os.environ["PIPENV_PYPI_MIRROR"] = str(pypi_mirror)
-        os.environ["PIPENV_VERBOSITY"] = str(project.s.PIPENV_VERBOSITY)
-        os.environ["PIPENV_REQ_DIR"] = req_dir
         os.environ["PIP_NO_INPUT"] = "1"
         pipenv_site_dir = get_pipenv_sitedir()
         if pipenv_site_dir is not None:
             os.environ["PIPENV_SITE_DIR"] = pipenv_site_dir
         else:
             os.environ.pop("PIPENV_SITE_DIR", None)
-        if keep_outdated:
-            os.environ["PIPENV_KEEP_OUTDATED"] = "1"
         with console.status("Locking...", spinner=project.s.PIPENV_SPINNER) as st:
             # This conversion is somewhat slow on local and file-type requirements since
             # we now download those requirements / make temporary folders to perform
             # dependency resolution on them, so we are including this step inside the
             # spinner context manager for the UX improvement
             st.console.print("Building requirements...")
-            deps = convert_deps_to_pip(deps, project, include_index=True)
+            deps = convert_deps_to_pip(deps, project)
             constraints = set(deps)
-            with tempfile.NamedTemporaryFile(
-                mode="w+", prefix="pipenv", suffix="constraints.txt", delete=False
-            ) as constraints_file:
-                constraints_file.write(str("\n".join(constraints)))
-            cmd.append("--constraints-file")
-            cmd.append(constraints_file.name)
             st.console.print("Resolving dependencies...")
-            c = resolve(cmd, st, project=project)
-            if c.returncode == 0:
-                st.console.print(environments.PIPENV_SPINNER_OK_TEXT.format("Success!"))
-                if not project.s.is_verbose() and c.stderr.strip():
-                    click.echo(click.style(f"Warning: {c.stderr.strip()}"), err=True)
-            else:
+            try:
+                results = resolve_packages(
+                    pre,
+                    clear,
+                    project.s.is_verbose(),
+                    allow_global,
+                    req_dir,
+                    packages=deps,
+                    category=category,
+                    constraints=constraints,
+                )
+                if results:
+                    st.console.print(
+                        environments.PIPENV_SPINNER_OK_TEXT.format("Success!")
+                    )
+            except Exception:
                 st.console.print(
                     environments.PIPENV_SPINNER_FAIL_TEXT.format("Locking Failed!")
                 )
-                click.echo(f"Output: {c.stdout.strip()}", err=True)
-                click.echo(f"Error: {c.stderr.strip()}", err=True)
-    try:
-        with open(target_file.name) as fh:
-            results = json.load(fh)
-    except (IndexError, json.JSONDecodeError):
-        click.echo(c.stdout.strip(), err=True)
-        click.echo(c.stderr.strip(), err=True)
-        if os.path.exists(target_file.name):
-            os.unlink(target_file.name)
-        raise RuntimeError("There was a problem with locking.")
-    if os.path.exists(target_file.name):
-        os.unlink(target_file.name)
+                raise RuntimeError("There was a problem with locking.")
     if lockfile_section not in lockfile:
         lockfile[lockfile_section] = {}
     return prepare_lockfile(results, pipfile, lockfile[lockfile_section])
@@ -1148,19 +1120,16 @@ def resolve_deps(
     """
     index_lookup = {}
     markers_lookup = {}
-    python_path = which("python", allow_global=allow_global)
     if not os.environ.get("PIP_SRC"):
         os.environ["PIP_SRC"] = project.virtualenv_src_location
-    backup_python_path = sys.executable
     results = []
     resolver = None
     if not deps:
         return results, resolver
     # First (proper) attempt:
-    req_dir = req_dir if req_dir else os.environ.get("req_dir", None)
     if not req_dir:
         req_dir = create_tracked_tempdir(prefix="pipenv-", suffix="-requirements")
-    with HackedPythonVersion(python_version=python, python_path=python_path):
+    with HackedPythonVersion(python_path=project.python(system=allow_global)):
         try:
             results, hashes, markers_lookup, resolver, skipped = actually_resolve_deps(
                 deps,
@@ -1179,8 +1148,7 @@ def resolve_deps(
     # Second (last-resort) attempt:
     if results is None:
         with HackedPythonVersion(
-            python_version=".".join([str(s) for s in sys.version_info[:3]]),
-            python_path=backup_python_path,
+            python_path=project.python(system=allow_global),
         ):
             try:
                 # Attempt to resolve again, with different Python version information,
